@@ -53,6 +53,7 @@ struct _GeditDocumentsPanelPrivate
 	GtkWidget          *treeview;
 	GtkTreeModel       *model;
 
+	guint               selection_changed_handler_id;
 	guint               refresh_idle_id;
 
 	guint               adding_tab : 1;
@@ -410,8 +411,13 @@ refresh_notebook_foreach (GeditNotebook       *notebook,
 static gboolean
 refresh_list_idle (GeditDocumentsPanel *panel)
 {
+	GtkTreeSelection *selection;
+
 	gedit_debug (DEBUG_PANEL);
 
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (panel->priv->treeview));
+	g_signal_handler_block (selection, panel->priv->selection_changed_handler_id);
+
 	gtk_tree_store_clear (GTK_TREE_STORE (panel->priv->model));
 
 	panel->priv->adding_tab = TRUE;
@@ -426,6 +432,8 @@ refresh_list_idle (GeditDocumentsPanel *panel)
 
 	panel->priv->refresh_idle_id = 0;
 
+	g_signal_handler_unblock (selection, panel->priv->selection_changed_handler_id);
+
 	return FALSE;
 }
 
@@ -587,16 +595,13 @@ set_window (GeditDocumentsPanel *panel,
 }
 
 static void
-treeview_cursor_changed (GtkTreeView         *view,
-			 GeditDocumentsPanel *panel)
+treeview_selection_changed (GtkTreeSelection    *selection,
+			    GeditDocumentsPanel *panel)
 {
 	GtkTreeIter iter;
-	GtkTreeSelection *selection;
 
 	gedit_debug (DEBUG_PANEL);
 
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (panel->priv->treeview));
-
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter))
 	{
 		GeditNotebook *notebook;
@@ -1164,13 +1169,12 @@ gedit_documents_panel_init (GeditDocumentsPanel *panel)
 	                  panel);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (panel->priv->treeview));
-
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+	panel->priv->selection_changed_handler_id = g_signal_connect (selection,
+								      "changed",
+								       G_CALLBACK (treeview_selection_changed),
+								       panel);
 
-	g_signal_connect (panel->priv->treeview,
-			  "cursor-changed",
-			  G_CALLBACK (treeview_cursor_changed),
-			  panel);
 	g_signal_connect (panel->priv->treeview,
 			  "button-press-event",
 			  G_CALLBACK (panel_button_press_event),
