@@ -30,7 +30,6 @@
 #include "gedit-utils.h"
 #include "gedit-animated-overlay.h"
 #include "gedit-floating-slider.h"
-#include "gedit-rounded-frame.h"
 
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
@@ -38,7 +37,6 @@
 
 #define GEDIT_VIEW_FRAME_SEARCH_DIALOG_TIMEOUT (30*1000) /* 30 seconds */
 
-#define MIN_SEARCH_COMPLETION_KEY_LEN	3
 #define SEARCH_POPUP_MARGIN 12
 
 #define GEDIT_VIEW_FRAME_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GEDIT_TYPE_VIEW_FRAME, GeditViewFramePrivate))
@@ -72,7 +70,6 @@ struct _GeditViewFramePrivate
 	guint        search_flags;
 
 	GtkWidget   *slider;
-	GtkWidget   *search_widget;
 	GtkWidget   *search_entry;
 	GtkCssProvider *search_css;
 
@@ -103,9 +100,6 @@ typedef enum
 	GEDIT_SEARCH_ENTRY_NOT_FOUND
 } GeditSearchEntryBgColor;
 
-/* The search entry completion is shared among all the views */
-GtkListStore *search_completion_model = NULL;
-
 G_DEFINE_TYPE_WITH_CODE (GeditViewFrame, gedit_view_frame, GTK_TYPE_BOX,
                          g_type_add_class_private (g_define_type_id, sizeof (GeditViewFrameClassPrivate)))
 
@@ -213,69 +207,6 @@ hide_search_widget (GeditViewFrame *frame,
 	                          frame->priv->search_entry_focus_out_id);
 }
 
-static void
-add_search_completion_entry (const gchar *str)
-{
-	gchar        *text;
-	gboolean      valid;
-	GtkTreeModel *model;
-	GtkTreeIter   iter;
-
-	if (str == NULL)
-		return;
-
-	text = gedit_utils_unescape_search_text (str);
-
-	if (g_utf8_strlen (text, -1) < MIN_SEARCH_COMPLETION_KEY_LEN)
-	{
-		g_free (text);
-		return;
-	}
-
-	g_return_if_fail (GTK_IS_TREE_MODEL (search_completion_model));
-
-	model = GTK_TREE_MODEL (search_completion_model);
-
-	/* Get the first iter in the list */
-	valid = gtk_tree_model_get_iter_first (model, &iter);
-
-	while (valid)
-	{
-		/* Walk through the list, reading each row */
-		gchar *str_data;
-
-		gtk_tree_model_get (model,
-		                    &iter,
-		                    0,
-		                    &str_data,
-		                    -1);
-
-		if (strcmp (text, str_data) == 0)
-		{
-			g_free (text);
-			g_free (str_data);
-			gtk_list_store_move_after (GTK_LIST_STORE (model),
-			                           &iter,
-			                           NULL);
-
-			return;
-		}
-
-		g_free (str_data);
-
-		valid = gtk_tree_model_iter_next (model, &iter);
-	}
-
-	gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
-	gtk_list_store_set (GTK_LIST_STORE (model),
-	                    &iter,
-	                    0,
-	                    text,
-	                    -1);
-
-	g_free (text);
-}
-
 static gboolean
 search_entry_flush_timeout (GeditViewFrame *frame)
 {
@@ -454,8 +385,6 @@ search_again (GeditViewFrame *frame,
 
 	entry_text = gtk_entry_get_text (GTK_ENTRY (frame->priv->search_entry));
 
-	add_search_completion_entry (entry_text);
-
 	run_search (frame,
 	            entry_text,
 	            search_backward,
@@ -833,85 +762,31 @@ search_entry_insert_text (GtkEditable    *editable,
 	}
 }
 
-static gboolean
-completion_func (GtkEntryCompletion *completion,
-                 const gchar        *key,
-		 GtkTreeIter        *iter,
-		 gpointer            data)
-{
-	gchar *item = NULL;
-	gboolean ret = FALSE;
-	GtkTreeModel *model;
-	GeditViewFramePrivate *priv = GEDIT_VIEW_FRAME (data)->priv;
-	const gchar *real_key;
-
-	if (priv->search_mode == GOTO_LINE)
-		return FALSE;
-
-	real_key = gtk_entry_get_text (GTK_ENTRY (gtk_entry_completion_get_entry (completion)));
-
-	if (g_utf8_strlen (real_key, -1) <= MIN_SEARCH_COMPLETION_KEY_LEN)
-		return FALSE;
-
-	model = gtk_entry_completion_get_model (completion);
-	g_return_val_if_fail (gtk_tree_model_get_column_type (model, 0) == G_TYPE_STRING,
-	                      FALSE);
-
-	gtk_tree_model_get (model,
-	                    iter,
-	                    0,
-	                    &item,
-	                    -1);
-
-	if (item == NULL)
-		return FALSE;
-
-	if (GEDIT_SEARCH_IS_CASE_SENSITIVE (priv->search_flags))
-	{
-		if (!strncmp (real_key, item, strlen (real_key)))
-			ret = TRUE;
-	}
-	else
-	{
-		gchar *normalized_string;
-		gchar *case_normalized_string;
-		
-		normalized_string = g_utf8_normalize (item, -1, G_NORMALIZE_ALL);
-		case_normalized_string = g_utf8_casefold (normalized_string, -1);
-
-		if (!strncmp (key, case_normalized_string, strlen (key)))
-			ret = TRUE;
-
-		g_free (normalized_string);
-		g_free (case_normalized_string);
-	}
-
-	g_free (item);
-
-	return ret;
-}
-
 static void
 customize_for_search_mode (GeditViewFrame *frame)
 {
+	GIcon *icon;
+
 	if (frame->priv->search_mode == SEARCH)
 	{
-		gtk_entry_set_icon_from_stock (GTK_ENTRY (frame->priv->search_entry),
-		                               GTK_ENTRY_ICON_PRIMARY,
-		                               GTK_STOCK_FIND);
+		icon = g_themed_icon_new_with_default_fallbacks ("edit-find-symbolic");
 
 		gtk_widget_set_tooltip_text (frame->priv->search_entry,
 		                             _("String you want to search for"));
 	}
 	else
 	{
-		gtk_entry_set_icon_from_stock (GTK_ENTRY (frame->priv->search_entry),
-		                               GTK_ENTRY_ICON_PRIMARY,
-		                               GTK_STOCK_JUMP_TO);
+		icon = g_themed_icon_new_with_default_fallbacks ("go-jump-symbolic");
 
 		gtk_widget_set_tooltip_text (frame->priv->search_entry,
 		                             _("Line you want to move the cursor to"));
 	}
+
+	gtk_entry_set_icon_from_gicon (GTK_ENTRY (frame->priv->search_entry),
+	                               GTK_ENTRY_ICON_PRIMARY,
+	                               icon);
+
+	g_object_unref (icon);
 }
 
 static void
@@ -1066,11 +941,9 @@ static GtkWidget *
 create_search_widget (GeditViewFrame *frame)
 {
 	GtkWidget          *search_widget;
-	GtkWidget          *hbox;
 	GtkWidget          *button;
-	GtkEntryCompletion *completion;
 
-	search_widget = gedit_rounded_frame_new ();
+	search_widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	gtk_widget_show (search_widget);
 
 	g_signal_connect (search_widget, "key-press-event",
@@ -1080,12 +953,6 @@ create_search_widget (GeditViewFrame *frame)
 	                  G_CALLBACK (search_widget_scroll_event),
 	                  frame);
 
-	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
-	gtk_widget_show (hbox);
-
-	gtk_container_add (GTK_CONTAINER (search_widget), hbox);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
-
 	/* add entry */
 	frame->priv->search_entry = gtk_entry_new ();
 	gtk_widget_show (frame->priv->search_entry);
@@ -1113,46 +980,16 @@ create_search_widget (GeditViewFrame *frame)
 		                  G_CALLBACK (search_entry_focus_out_event),
 		                  frame);
 
-	gtk_container_add (GTK_CONTAINER (hbox),
+	gtk_container_add (GTK_CONTAINER (search_widget),
 	                   frame->priv->search_entry);
 
-	if (search_completion_model == NULL)
-	{
-		/* Create a tree model and use it as the completion model */
-		search_completion_model = gtk_list_store_new (1, G_TYPE_STRING);
-	}
-
-	/* Create the completion object for the search entry */
-	completion = gtk_entry_completion_new ();
-	gtk_entry_completion_set_model (completion,
-	                                GTK_TREE_MODEL (search_completion_model));
-
-	/* Use model column 0 as the text column */
-	gtk_entry_completion_set_text_column (completion, 0);
-
-	gtk_entry_completion_set_minimum_key_length (completion,
-	                                             MIN_SEARCH_COMPLETION_KEY_LEN);
-
-	gtk_entry_completion_set_popup_completion (completion, FALSE);
-	gtk_entry_completion_set_inline_completion (completion, TRUE);
-
-	gtk_entry_completion_set_match_func (completion,
-	                                     completion_func,
-	                                     frame,
-	                                     NULL);
-
-	/* Assign the completion to the entry */
-	gtk_entry_set_completion (GTK_ENTRY (frame->priv->search_entry),
-	                          completion);
-	g_object_unref (completion);
-
 	customize_for_search_mode (frame);
 
 	/* Up/Down buttons */
 	button = create_button_from_stock (GTK_STOCK_GO_UP);
 	gtk_widget_show (button);
 
-	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (search_widget), button, FALSE, FALSE, 0);
 
 	g_signal_connect (button,
 	                  "clicked",
@@ -1162,7 +999,7 @@ create_search_widget (GeditViewFrame *frame)
 	button = create_button_from_stock (GTK_STOCK_GO_DOWN);
 	gtk_widget_show (button);
 
-	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (search_widget), button, FALSE, FALSE, 0);
 
 	g_signal_connect (button,
 	                  "clicked",
@@ -1260,7 +1097,6 @@ init_search_entry (GeditViewFrame *frame)
 		{
 			g_free (frame->priv->old_search_text);
 			frame->priv->old_search_text = old_find_text;
-			add_search_completion_entry (old_find_text);
 			g_signal_handler_block (frame->priv->search_entry,
 			                        frame->priv->search_entry_changed_id);
 
@@ -1284,6 +1120,12 @@ start_interactive_search_real (GeditViewFrame *frame)
 	GtkTextIter iter;
 	GtkTextMark *mark;
 
+	if (gtk_bin_get_child (GTK_BIN (frame->priv->slider)) == NULL)
+	{
+		gtk_container_add (GTK_CONTAINER (frame->priv->slider),
+		                   create_search_widget (frame));
+	}
+
 	if (gtk_widget_get_visible (frame->priv->slider))
 	{
 		if (frame->priv->search_mode != frame->priv->request_search_mode)
@@ -1434,6 +1276,7 @@ gedit_view_frame_init (GeditViewFrame *frame)
 						     frame);
 
 	frame->priv->view = gedit_view_new (doc);
+	gtk_widget_set_vexpand (frame->priv->view, TRUE);
 	gtk_widget_show (frame->priv->view);
 
 	g_object_unref (doc);
@@ -1456,13 +1299,19 @@ gedit_view_frame_init (GeditViewFrame *frame)
 	gtk_box_pack_start (GTK_BOX (frame), frame->priv->overlay, TRUE, TRUE, 0);
 
 	/* Add slider */
-	frame->priv->search_widget = create_search_widget (frame);
 	frame->priv->slider = gedit_floating_slider_new ();
-	gtk_container_add (GTK_CONTAINER (frame->priv->slider),
-	                   frame->priv->search_widget);
 	gtk_widget_set_halign (frame->priv->slider, GTK_ALIGN_END);
 	gtk_widget_set_valign (frame->priv->slider, GTK_ALIGN_START);
-	gtk_widget_set_margin_right (frame->priv->slider, SEARCH_POPUP_MARGIN);
+
+	if (gtk_widget_get_direction (frame->priv->slider) == GTK_TEXT_DIR_LTR)
+	{
+		gtk_widget_set_margin_right (frame->priv->slider, SEARCH_POPUP_MARGIN);
+	}
+	else
+	{
+		gtk_widget_set_margin_left (frame->priv->slider, SEARCH_POPUP_MARGIN);
+	}
+
 	g_object_set (G_OBJECT (frame->priv->slider),
 	              "easing", GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_IN_OUT,
 	              "blocking", GEDIT_THEATRICS_CHOREOGRAPHER_BLOCKING_DOWNSTAGE,
